@@ -15,6 +15,7 @@
 namespace APP\migration\install;
 
 use Illuminate\Database\Schema\Blueprint;
+use Illuminate\Support\Facades\DB;
 use Illuminate\Support\Facades\Schema as Schema;
 
 class MetricsMigration extends \PKP\migration\Migration
@@ -28,7 +29,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->comment('Daily statistics for views of the homepage.');
             $table->bigIncrements('metrics_context_id');
 
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
             $table->index(['load_id'], 'metrics_context_load_id');
 
             $table->bigInteger('context_id');
@@ -42,7 +43,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->comment('Daily statistics for views of published submissions in each series.');
             $table->bigIncrements('metrics_series_id');
 
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
             $table->index(['load_id'], 'metrics_series_load_id');
 
             $table->bigInteger('context_id');
@@ -62,7 +63,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->comment('Daily statistics for views and downloads of published submissions and files.');
             $table->bigIncrements('metrics_submission_id');
 
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
             $table->index(['load_id'], 'ms_load_id');
 
             $table->bigInteger('context_id');
@@ -96,7 +97,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->comment('Daily statistics matching the COUNTER R5 protocol for views and downloads of published submissions and files.');
             $table->bigIncrements('metrics_counter_submission_daily_id');
 
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
             $table->index(['load_id'], 'msd_load_id');
 
             $table->bigInteger('context_id');
@@ -153,7 +154,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->comment('Daily statistics matching the COUNTER R5 protocol for views and downloads from institutions.');
             $table->bigIncrements('metrics_counter_submission_institution_daily_id');
 
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
             $table->index(['load_id'], 'msid_load_id');
 
             $table->bigInteger('context_id');
@@ -218,7 +219,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->comment('Daily statistics by country, region and city for views and downloads of published submissions and files.');
             $table->bigIncrements('metrics_submission_geo_daily_id');
 
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
             $table->index(['load_id'], 'msgd_load_id');
 
             $table->bigInteger('context_id');
@@ -237,7 +238,16 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->integer('metric_unique');
 
             $table->index(['context_id', 'submission_id'], 'msgd_context_id_submission_id');
-            $table->unique(['load_id', 'context_id', 'submission_id', 'country', 'region', 'city', 'date'], 'msgd_uc_load_context_submission_c_r_c_date');
+            switch (DB::getDriverName()) {
+                case 'mysql':
+                    // See "Create a database table" here: https://db-ip.com/db/format/ip-to-city-lite/csv.html
+                    // where city is defined as varchar(80)
+                    $table->unique([DB::raw('load_id, context_id, submission_id, country, region, city(80), date')], 'msgd_uc_load_context_submission_c_r_c_date');
+                    break;
+                case 'pgsql':
+                    $table->unique(['load_id', 'context_id', 'submission_id', 'country', 'region', 'city', 'date'], 'msgd_uc_load_context_submission_c_r_c_date');
+                    break;
+            }
         });
         Schema::create('metrics_submission_geo_monthly', function (Blueprint $table) {
             $table->comment('Monthly statistics by country, region and city for views and downloads of published submissions and files.');
@@ -259,7 +269,16 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->integer('metric_unique');
 
             $table->index(['context_id', 'submission_id'], 'msgm_context_id_submission_id');
-            $table->unique(['context_id', 'submission_id', 'country', 'region', 'city', 'month'], 'msgm_uc_context_submission_c_r_c_month');
+            switch (DB::getDriverName()) {
+                case 'mysql':
+                    // See "Create a database table" here: https://db-ip.com/db/format/ip-to-city-lite/csv.html
+                    // where city is defined as varchar(80)
+                    $table->unique([DB::raw('context_id, submission_id, country, region, city(80), month')], 'msgm_uc_context_submission_c_r_c_month');
+                    break;
+                case 'pgsql':
+                    $table->unique(['context_id', 'submission_id', 'country', 'region', 'city', 'month'], 'msgm_uc_context_submission_c_r_c_month');
+                    break;
+            }
         });
         // Usage stats total book and chapter item temporary records
         Schema::create('usage_stats_total_temporary_records', function (Blueprint $table) {
@@ -301,7 +320,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->string('country', 2)->default('');
             $table->string('region', 3)->default('');
             $table->string('city', 255)->default('');
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
         });
         // Usage stats unique book and chapter item investigations temporary records
         // No need to consider series_id here because investigations are only relevant/calculated on submission level.
@@ -339,7 +358,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->string('country', 2)->default('');
             $table->string('region', 3)->default('');
             $table->string('city', 255)->default('');
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
         });
         // Usage stats unique book and chapter item requests temporary records
         // No need to consider series_id here because requests are only relevant/calculated on submission level.
@@ -377,7 +396,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->string('country', 2)->default('');
             $table->string('region', 3)->default('');
             $table->string('city', 255)->default('');
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
         });
         // Usage stats unique title investigations temporary records
         // No need to consider series_id here because investigations are only relevant/calculated on submission level.
@@ -414,7 +433,7 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->string('country', 2)->default('');
             $table->string('region', 3)->default('');
             $table->string('city', 255)->default('');
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
         });
         // Usage stats unique title requests temporary records
         // No need to consider series_id here because requests are only relevant/calculated on submission level.
@@ -452,14 +471,14 @@ class MetricsMigration extends \PKP\migration\Migration
             $table->string('country', 2)->default('');
             $table->string('region', 3)->default('');
             $table->string('city', 255)->default('');
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
         });
         // Usage stats institution temporary records
         Schema::create('usage_stats_institution_temporary_records', function (Blueprint $table) {
             $table->comment('Temporary stats by institution. Data in this table is provisional. See the metrics_* tables for compiled stats.');
             $table->bigIncrements('usage_stats_temp_institution_id');
 
-            $table->string('load_id', 255);
+            $table->string('load_id', 50);
             $table->bigInteger('line_number');
 
             $table->bigInteger('institution_id');
