@@ -354,7 +354,11 @@ class Mailable extends IlluminateMailable
     protected function buildSubject($message): self
     {
         $this->subject ??= ''; // Allow email with empty subject if not set
-        $subject = app('mailer')->compileParams($this->subject, $this->viewData);
+        $withoutTagViewData = collect($this->viewData)
+            ->map(fn(mixed $viewableData) => is_string($viewableData) ? strip_tags($viewableData) : $viewableData)
+            ->toArray();
+            
+        $subject = app('mailer')->compileParams($this->subject, $withoutTagViewData);
         if (empty($subject)) {
             trigger_error(
                 'You are sending ' . static::getName() ?? static::class . ' email with empty subject',
