@@ -338,7 +338,8 @@ abstract class PreflightCheckMigration extends \PKP\migration\Migration
         );
 
         if (count($result) > 0) {
-            $tableNames = data_get($result, '*.TABLE_NAME');
+            error_log(print_r($result,true));
+            $tableNames = data_get($result, '*.table_name');
             throw new Exception(
                 'Storage engine that doesn\'t support foreign key constraints detected in one or more tables: ' .
                 implode(', ', $tableNames) . '. Change to InnoDB before running the upgrade.'
